package purejavacomm.testsuite;

import java.nio.ByteBuffer;

import com.sun.jna.Native;
import com.sun.jna.NativeLibrary;
import com.sun.jna.NativeLong;
import com.sun.jna.Pointer;

public class Benchmark {
	static CLibDM m_CLibDM;
	static CLib m_CLibIB;
	static CLib m_CLib;
	static byte[] m_Bytes = new byte[4096];
	static ByteBuffer m_BytesAsBuffer = ByteBuffer.wrap(m_Bytes);
	static long m_BytesN = m_Bytes.length;

	public interface CLib extends com.sun.jna.Library {
/* 1 */		int htonl(int x);
/* 2 */		void memset(byte[] p, int c, long len);
/* 3 */		void memmove(byte[] d,byte[] s, long len);
	}
	
	public static class CLibDM implements CLib {
/* 4 */		native public int htonl(int x);
/* 5 */		native public void memset(byte[] p, int c, long len);
/* 6 */		native public void memmove(byte[] d,byte[] s, long len);
	}

	static void call_memset() {
		m_CLib.memset(m_Bytes, 69, m_BytesN);
	}

	static void call_memmove() {
		m_CLib.memmove(m_Bytes,m_Bytes, m_BytesN);
	}

	static int call_htonl() {
		return m_CLib.htonl(0xDEADBEEF);
	}

	public static void runBenchmarks(int N, double[] res, int offs) {
		long t0, dt;
		t0 = System.nanoTime();
		for (int i = 0; i < N; i++)
			call_htonl();
		res[offs] = (System.nanoTime() - t0) / N;

		t0 = System.nanoTime();
		for (int i = 0; i < N; i++)
			call_memset();
		dt = System.nanoTime() - t0;
		res[offs + 1] = (System.nanoTime() - t0) / N;
		
		t0 = System.nanoTime();
		for (int i = 0; i < N; i++)
			call_memmove();
		dt = System.nanoTime() - t0;
		res[offs + 2] = (System.nanoTime() - t0) / N;
	}

	public static void main(String[] args) {
		try { 
			Native.register(CLibDM.class, NativeLibrary.getInstance("c"));
			m_CLibDM = new CLibDM();
			m_CLibIB = (CLib) Native.loadLibrary("c", CLib.class);

			int N = 10000;
			double res[] = new double[6];
			for (int i = 0; i < 10000; i++) {
				
				m_CLib = m_CLibIB;
				runBenchmarks(N, res, 0);

				m_CLib = m_CLibDM;
				runBenchmarks(N, res, 3);
				
				for (int j = 0; j < res.length; ++j)
					System.out.printf("%15.3f", res[j] / 1000.0);
				System.out.println();
			}
			System.out.printf("%15s%15s%150s%15s%15s%15s\n", "IB/htonl", "IB/memset","IB/memmove", "DM/htonl", "DM/htonl", "DM/memmove");
			System.out.printf("results in usec");
		} catch (Throwable t) {
			t.printStackTrace();
		}

	}
}
